c getacc_ppn.f
c Parametrized post-newtonian accelerations (to be added).
c Miroslav Broz (miroslav.broz@email.cz), May 26th 2016

c Reference: Fabrycky (2010), Eq. (2)
c
c f_GR = -G(m_star + m_p)/(r^2 c^2) * ( -2(2-eta)\dot r \dot\vec r + [ (1+3eta)\dot\vec r\cdot\dot\vec r - 3/2 eta(\dot r)^2 - 2(2+eta) G(m_star+m_p)/r ] \hat r )

      subroutine getacc_ppn(nbod,mass,xb,yb,zb,vxb,vyb,vzb,axb,ayb,azb)

      include "../swift.inc"
      include "../misc/const.inc"
      include "tides.inc"

c input
      integer nbod
      real*8 mass(nbod)
      real*8 xb(nbod),yb(nbod),zb(nbod)
      real*8 vxb(nbod),vyb(nbod),vzb(nbod)

c output
      real*8 axb(nbod),ayb(nbod),azb(nbod)

c internal
      integer i, j
      real*8 xij, yij, zij, rij2, rij, vij2, dotr2, dotr
      real*8 vxij, vyij, vzij
      real*8 eta, t1, t2, t3, t4, t5, c_
      real*8 acc, ax, ay, az, fac

      if (.not.use_ppn) return

      c_ = c/AU*day

      do i = 1, nbod  ! "planet"
        do j = i+1, nbod  ! "star"

          xij = xb(i) - xb(j)
          yij = yb(i) - yb(j)
          zij = zb(i) - zb(j)
          vxij = vxb(i) - vxb(j)
          vyij = vyb(i) - vyb(j)
          vzij = vzb(i) - vzb(j)

          rij2 = xij*xij + yij*yij + zij*zij
          rij = sqrt(rij2)
          vij2 = vxij*vxij + vyij*vyij + vzij*vzij
          dotr = (xij*vxij + yij*vyij + zij*vzij)/rij
          dotr2 = dotr*dotr

          eta = mass(j)*mass(i)/(mass(j)+mass(i))**2
          t1 = (mass(j)+mass(i))/(rij2*c_**2)
          t2 = -2.d0*(2.d0-eta)*dotr
          t3 = (1.d0+3.0d0*eta)*vij2
          t4 = -3.d0/2.d0*eta*dotr2
          t5 = -2.d0*(2.d0+eta)*(mass(j)+mass(i))/rij

          ax = t1*(t2*vxij + (t3+t4+t5)*xij/rij)
          ay = t1*(t2*vyij + (t3+t4+t5)*yij/rij)
          az = t1*(t2*vzij + (t3+t4+t5)*zij/rij)

          axb(i) = axb(i) - ax
          ayb(i) = ayb(i) - ay
          azb(i) = azb(i) - az

        enddo
      enddo

      return
      end


